/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * main backup routines for backing up a single file.
 * set up as an objectHandler to enable working
 * with Util.recurseDirectory
 * 
 * @author  Georg Dietrich
 * @version 1.1 on 2004-04-14
 * @version 1.0 on 2004-04-09
 */
public class Backup implements ObjectHandler {

	protected Settings settings;

	/**
	 * constructor saves the settings object for further reference
	 * 
	 * @param mySettings  the backup settings to use
	 */
	public Backup (Settings mySettings) {
		settings = mySettings;
	}
	
	/**
	 * main backup routine, gets called for all files which might be backed up.
	 * returns false as soon as the "don't continue" flag is set.
	 * 
	 * @param object  the single file to back up
	 */
	public boolean perform(Object oSource) {

		// check whether "don't continue" flag is set
		if (!settings.backupFinished) return false;

		File source  = (File) oSource;
		File backup  = Util.source2backup(source, settings);
		File data    = Util.source2data(source, settings);

		// generate current file's hash sum
		int hashSum = Util.hashSum(source);
		
		// file precheck

		// scan the data file
		DataFile dataFile = new DataFile(data);
		
		// if data file does not exist or has length of 0, backup
		if (data.length() == 0L) { backup (source, backup, dataFile, hashSum);	return true; }

		// if backup conditions apply, back up and return.		
		switch (settings.backupType) {
			case 1:
			if (hashSum!=dataFile.incHash) {
				backup (source, backup, dataFile, hashSum);
				return true;
			}
			break;
			
			case 2:
			if (hashSum!=dataFile.difHash) {
				backup (source, backup, dataFile, hashSum);
				return true;
			}
			break;
			
			case 3:
			if (settings.earlierBackups.contains(dataFile.difDate)) {
				if (hashSum!=dataFile.difHash) {
					backup (source, backup, dataFile, hashSum);
					return true;
				}
				break;
			}
			if (hashSum!=dataFile.fulHash) {
				backup (source, backup, dataFile, hashSum);
				return true;
			}
			break;
			
			case 4:
			if (!settings.earlierBackups.contains(dataFile.fulDate)) {
				backup (source, backup, dataFile, hashSum);
				return true;
			}
			if (hashSum!=dataFile.fulHash) {
				backup (source, backup, dataFile, hashSum);
				return true;
			}
			break;
			
			default:
			throw new RuntimeException();
		}
		
		// update ui - increase number of skipped files
		System.out.print(".");
		ProgressInterface.display(settings.currentSize, settings.backupSize, settings.filesBackedUp, settings.filesSkipped++);

		return true;
		
	}

	/**
	 * backup one file to zip
	 * 
	 * @param source
	 * @param backup
	 * @param data
	 */
	void backup (File source, File backup, DataFile dataFile, int hashSum) {

		// check whether backup size limit is reached. YES -> set flag and don't backup.
		if (settings.currentSize >= settings.backupSize) { settings.backupFinished = false; return; } 

		// assure the necessary directories are present
		Util.makeDirs(backup);

		// write the zip file
		try {
			
			FileOutputStream fileout = new FileOutputStream(backup);
			ZipOutputStream  zipout  = new ZipOutputStream (fileout);
			FileInputStream  filein  = new FileInputStream (source);
			byte[]           buffer  = new byte[1024];

			zipout.setLevel(9);
			zipout.putNextEntry(new ZipEntry(source.getName()));

			for (int read = filein.read(buffer); read != -1; read = filein.read(buffer)) {
				zipout.write(buffer,0,read);
			}

			zipout.closeEntry();
			zipout.close();
			fileout.close();
			filein.close();

		} catch (Exception e) {
			System.err.println ("problem during backup of file "+source);
			e.printStackTrace();
			throw new RuntimeException();
		}
		
		// add the zip file's length to the current length
		settings.currentSize += backup.length();
		
		// store updates in data file
		switch (settings.backupType) {
			case 4:
			dataFile.fulHash = hashSum;
			dataFile.fulDate = settings.backupTime;
			case 3:
			case 2:
			dataFile.difHash = hashSum;
			dataFile.difDate = settings.backupTime;
			case 1:
			dataFile.incHash = hashSum;
			dataFile.incDate = settings.backupTime;
			dataFile.save();
			break;
			
			default:
			throw new RuntimeException();
		}
		
		// print status
		ProgressInterface.display(settings.currentSize, settings.backupSize, settings.filesBackedUp++, settings.filesSkipped);
		System.out.println (settings.currentSize+" of "+settings.backupSize+" with "+backup);
	}
}
