/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package backup;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;

/**
 * a collection of static utility methods for the backup system
 * 
 * @author  Georg Dietrich
 * @version 1.0 on 2004-04-08
 */
public class Util {

	/**
	 * generate a hash sum out of a file's content
	 * 
	 * @param source  the source file
	 * @return        the hash sum
	 */
	public static int hashSum (File file) {
		try {
			
			int hash = 0;			
			FileInputStream filein  = new FileInputStream (file);
			byte[] buffer = new byte[1024];
			for (int read = filein.read(buffer); read != -1; read = filein.read(buffer)) {
				for (int i = 0; i < read; i ++) {
					hash += 91;
					hash ^= buffer[i];
					if (hash > 1E10) hash -= 1E9;
				}
			}
			return hash;

		} catch (Exception e) {
			System.err.println("hashSum gescheitert fr "+file);
			e.printStackTrace();
			throw new RuntimeException();
		}
	}

	/**
	 * construct a backup file reference from a source reference
	 */
	public static File source2backup (File source, Settings settings) {
		String path = canonicalPath(source);
		return new File(settings.backupRootPath+path.substring(settings.sourceRootPath.length())+".zip");
	}

	/**
	 * construct a data file reference from a source reference
	 */
	public static File source2data (File source, Settings settings) {
		String path = canonicalPath(source);
		return new File(settings.dataRootPath+path.substring(settings.sourceRootPath.length())+".txt");
	}

	/**
	 * construct a source file reference from a data reference
	 */
	public static File data2source (File data, Settings settings) {
		String path = canonicalPath(data);
		return new File(settings.sourceRootPath+path.substring(settings.dataRootPath.length(),path.length()-4));
	}

	/**
	 * parse a string into an int
	 * 
	 * @param string  the string to parse
	 * @return        the integer parsed
	 */
	public static int parseInt (String string) {
		try {
			return NumberFormat.getInstance().parse(string).intValue();
		} catch (ParseException e) {
			System.err.println("parseInt gescheitert fr "+string);
			e.printStackTrace();
			throw new RuntimeException();
		}
	}

	/**
	 * recurses a directory tree performing actions on all files, not on dirs
	 * 
	 * @param target  the path to start with in the recursive search
	 */
	public static void recurseDirectory(File target, ObjectHandler handler) {
		if (target.isDirectory()) {
			File[] directoryContent = target.listFiles();
			for (int i = 0; i < directoryContent.length; i++) {
				recurseDirectory(directoryContent[i], handler);
			}
		} else {
			handler.perform(target);
		}
	}

	/**
	 * recurses a directory tree performing actions on all files, not on dirs
	 * but first filters file and dir names
	 * 
	 * @param target  the path to start with in the recursive search
	 * @return  false if recursion should be stopped
	 */
	public static boolean recurseDirectoryFilter(File target, ObjectHandler handler, Settings settings) {

		// filter
		String name = target.getName();
		for (Enumeration e = settings.filter.elements(); e.hasMoreElements(); ) {
			if (name.matches(e.nextElement().toString()) ^ !settings.filterOut) return true;
		}
		
		// recurse / perform
		if (target.isDirectory()) {
			File[] directoryContent = target.listFiles();
			for (int i = 0; i < directoryContent.length; i++) {
				if (!recurseDirectoryFilter(directoryContent[i], handler, settings)) return false;
			}
			return true;
		} else {
			return handler.perform(target);
		}
	}

	/**
	 * get the canonical path for a file
	 * 
	 * @param file  the file to get the canonical path for
	 * @return      the canonical path of the file
	 */
	public static String canonicalPath (File file) {
		try {
			return file.getCanonicalPath();
		} catch (IOException e) {
			System.err.println("canonicalPath gescheitert fr "+file);
			throw new RuntimeException();
		}
	}

	/**
	 * if the directories leading to an abstract file location do not exist,
	 * create them
	 * 
	 * @param file  the file location to create the directories for
	 */
	public static void makeDirs (File file) {
		if (file.getParentFile().exists()) {
			return;
		}
		if (!file.getParentFile().mkdirs()) {
			System.err.println("makedirs gescheitert fr "+file);
			throw new RuntimeException();
		}
	}
	
	/**
	 * get a string like 1972_12_10_08_34_27+200 for the current time including time zone
	 * 
	 * @return  a time string for the current time
	 */
	public static String getTimeString () {
		return new SimpleDateFormat("yyyy_MM_dd_HH_mm_ssZ").format(Calendar.getInstance().getTime());
	}

}
