/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package ui;

import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;

import backup.CoreMain;
import backup.Util;

/**
 * @author  Georg Dietrich
 * @version 1.2 on 2004-04-13
 * @version 1.1 on 2004-04-11
 * @version 1.0 on 2004-04-10
 */
public class GuiMain {

	// important: leave the variables in this order,
	// because they reference each other during their
	// instantiation

	/** the command line arguments */
	protected static String[] cmdArgs;

	/** the settings evaluated from command line and handed over to the core */
	protected GuiSettings settings = new GuiSettings(cmdArgs);

	/** flag whether to issue a warning on closing the main window */
	protected boolean runningWarning = false;

	/** the main application's main frame */
	protected JFrame mainFrame = null;
	
	/** the text field for editing/displaying the source path */
	protected JTextField sourceField = new JTextField(settings.source);

	/** the text field for editing/displaying the backup root path */
	protected JTextField backupField = new JTextField(settings.backupRoot);

	/** the text field for editing/displaying the data root path */
	protected JTextField dataField = new JTextField(settings.dataRoot);

	/** the text field for editing/displaying the file filters */
	protected JTextField filterField = new JTextField(settings.filters);

	/** the text field for editing/displaying the backup size */
	protected JTextField sizeField = new JTextField(settings.size);

	/** the button to open source path chooser */
	protected JButton sourceButton = new DirChooserButton("choose",sourceField);

	/** the button to open backup root path chooser */
	protected JButton backupButton = new DirChooserButton("choose",backupField);

	/** the button to open data root path chooser */
	protected JButton dataButton = new DirChooserButton("choose",dataField);

	/** the dropdown chooser for selecting the backup type */
	protected String[] types = {"    Choose backup type",
		"    Incremental","    Differential 2 Diff",
		"    Differential 2 Full","    Full"};
	protected JComboBox typeChooser = new TypeChooser(types, settings.backupType);

	/** the components to disable on starting the backup process */
	protected JComponent[] disableRunning = { sourceField, backupField, dataField, filterField,
																						sizeField, sourceButton, backupButton, dataButton,
																						typeChooser };

	/** the start / status / finished button */
	protected JButton startButton = getStartButton(settings);

	/** the progress bar */
	protected JProgressBar progressBar = getProgressBar();


	/** main application entry point */
	public static void main(String[] args) {
		// save the command line arguments for evaluation
		cmdArgs = args;
		// construct main application
		new GuiMain();
	}

	/**
	 * the main application class.  Careful - most variables are initialized on
	 * construction of this class!
	 */
	GuiMain () {
		
		// the main frame
		mainFrame = new JFrame();
		mainFrame.setSize(700, 300);
		mainFrame.setContentPane(getJContentPane(settings));
		mainFrame.setTitle("Georgs Java Backup Utility");
		mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		mainFrame.setVisible(true);
		
		// it seems this has to be set after the frame is made visible
		// although should it already have been initialized with the
		// backupType value.
		typeChooser.setSelectedIndex(settings.backupType);
		
		// window exit routine issuing a warning if backup process is running
		mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				warnBeforeExiting();
			}
		});

	}

	/**
	 * first time calles issues a warning.  Second time exits the program.
	 */
	void warnBeforeExiting() {
		if (runningWarning) {
			runningWarning = false;
			JFrame text = new TextFrame ("Warning",
			  "By Closing this window you interrupt the\n"+
			  "currently running backup process.\n"+
				"If you interrupt a running backup process\n"+
				"the backup data become inconsistent and not\n"+
				"all files may be backed up as you intended.\n"+
				"If you really interrupt this backup process,\n"+
				"start over with a new full backup.");
			Point loc = mainFrame.getLocation();
			loc.y += mainFrame.getHeight();
			text.setLocation(loc);
		} else {
			System.exit(0);
		}
	}

	/**
	 * get the main content panel
	 * 
	 * @param settings
	 * @return
	 */
	private JPanel getJContentPane(GuiSettings settings) {
		
		// create and layout panel (6 rows, 3 columns, space between rows/cols, border)
		JPanel mainPane = new JPanel();
		mainPane.setLayout(new java.awt.GridLayout(6,3,6,6));
		mainPane.setBorder(BorderFactory.createTitledBorder("backup settings"));

		// add content objects to the panel
		// row 1
		mainPane.add(new TextPanel("Source Direcory:"));
		mainPane.add(sourceField);
		mainPane.add(sourceButton);

		// row 2
		mainPane.add(new TextPanel("Destination Dir.:"));
		mainPane.add(backupField);
		mainPane.add(backupButton);

		// row 3
		mainPane.add(new TextPanel("Data Directory:"));
		mainPane.add(dataField);
		mainPane.add(dataButton);

		// row 4
		mainPane.add(new TextPanel("Backup Size"));
		mainPane.add(sizeField);
		mainPane.add(new TextPanel("Megabytes"));

		// row 5
		mainPane.add(new TextPanel("File Filter / Backup Type"));
		mainPane.add(filterField);
		mainPane.add(typeChooser);

		// row 6
		mainPane.add(getHelpButton());
		mainPane.add(progressBar);
		mainPane.add(startButton);

		return mainPane;
	}

	/**
	 * TextPanel is a JPanel displaying a text string
	 * 
	 * @author Georg Dietrich
	 */
	class TextPanel extends JPanel {
		TextPanel (String text) {
			JLabel filler = new JLabel(text);
			filler.setHorizontalAlignment(JLabel.CENTER);
			setLayout(new GridLayout(1, 1));
			add(filler);
		}
	}

	/**
	 * TypeChooser is the JComboBox dropdown chooser with custom renderer,
	 * and predefined text entries.
	 * 
	 * @author Georg Dietrich
	 */	
	class TypeChooser extends JComboBox {
		TypeChooser (String[] types, int initialType) {
			super(types);
			typeChooser = new JComboBox(types); 
			typeChooser.setSelectedIndex(initialType);
		}
	}

	/**
	 * DirChooserButton is a JButton that opens a directory chooser dialog
	 * and feeds the result into the text field of a JTextField.
	 * 
	 * @author Georg Dietrich
	 */	
	class DirChooserButton extends JButton {
		// constructor
		DirChooserButton (String title, final JTextField field) {
			super(title);
			// action listener for pressing the button
			addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					// display a file chooser that accepts only dirs
					JFileChooser fc = new JFileChooser(sourceField.getText());
					fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
					// check whether "OK" or "CANCEL" was pressed
					int returnVal = fc.showOpenDialog(null);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						// feed the chosen file into the appropriate text field
						field.setText(fc.getSelectedFile().toString());
					}
				}
			});
		}
	}

	/**
	 * @return  a help button that displays the help file
	 */
	private JButton getHelpButton() {

		// create the help button
		JButton help = new JButton();
		help.setText("Help");

		// set up the help action
		help.addActionListener(new java.awt.event.ActionListener() { 
			public void actionPerformed(java.awt.event.ActionEvent e) {

				// find the help directory
				String path = System.getProperty("java.class.path");
				File pFile = new File (path);
				if (!pFile.isDirectory()) pFile = pFile.getParentFile();
				path = Util.canonicalPath(pFile);
				
				// display the help file
				JFrame text = new TextFrame("Help me!", new File(path+File.separator+"help"+File.separator+"help.txt"));
				Point loc = mainFrame.getLocation();
				loc.y += mainFrame.getHeight();
				text.setLocation(loc);
			}
		});

		return help;
	}

	/**
	 * @return  a percentile progress bar with text display
	 */
	private JProgressBar getProgressBar() {
		JProgressBar bar = new JProgressBar(0,100);
		bar.setStringPainted(true);
		bar.setString("0 skipped / 0 backed up");
		return bar;
	}

	/**
	 * the start button is a multifunctional button that
	 * changes button text and function according to the
	 * program state:
	 * initially the button serves for starting the
	 * backup process.  during backup the button serves
	 * to interrupt the backup (same functionality as
	 * the close button on the upper right corner).
	 * after backup the button becomes an ordinary
	 * close button.
	 * 
	 * @param settings
	 * @return
	 */
	private JButton getStartButton(final GuiSettings settings) {
		
		// create the button
		final JButton button = new JButton();
		button.setText("start");
		
		// add action listener listening for button activation
		button.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) {
				
//				// if backup process is finished, exit
//				if (button.getText().equals("Finished - Exit")) {
//					System.exit(0);
//				}
				
				// if backup process is running, warn before exiting
				if (button.getText().equals("Running - Interrupt")) {
					warnBeforeExiting();
					return;
				}

				// copy gui values to settings
				settings.source     = sourceField.getText();
				settings.backupRoot = backupField.getText();
				settings.dataRoot   = dataField.getText();
				settings.backupType = typeChooser.getSelectedIndex();
				settings.size       = sizeField.getText();
				settings.filters    = filterField.getText();
				
				// check whether to start backup process
				if (settings.check()) {

					// redirect button use
					runningWarning = true;
					button.setText("Running - Interrupt");
					
					// disable all settings fields and buttons
					for (int i = 0; i < disableRunning.length; i++) {
						disableRunning[i].setEnabled(false);
					}

					// enable the graphical progress interface			
					ProgressInterface.inter = new ProgressInterface(progressBar);

					// set up new swing worker for the backup process
					SwingWorker worker = new SwingWorker () {

						public Object construct() {
							// construct the backup core
							new CoreMain(settings);
							return null;
						}

						public void finished () {

							// finished - re-enable the start button
							button.setText("start again");
							runningWarning = false;

							// enable all settings fields and buttons
							for (int i = 0; i < disableRunning.length; i++) {
								disableRunning[i].setEnabled(true);
							}

							// display finishing message
							JFrame text = new TextFrame ("Backup result", settings.result);
							Point loc = mainFrame.getLocation();
							loc.y += mainFrame.getHeight();
							text.setLocation(loc);
						}
					};
					// start backup process
					worker.start();
				}
			}
		});
		return button;
	}
	
}
