/*
 * jabak Java Backup
 * 
 * Copyright (C) under the terms of the GNU General Public License
 * as explained in the file "disclaimer.txt"
 */

package ui;

import java.io.File;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import backup.Util;

/**
 * Settings set within the GUI which are used in the backup core.
 * 
 * @author  Georg Dietrich
 * @version 1.0 on 2004-04-12
 */
public class GuiSettings {
	
	/** the source directory path */
	public String source = "choose source directory";

	/** the backup root directory path */
	public String backupRoot = "choose backup root directory";
	
	/** the data root directory path */
	public String dataRoot = "choose data root directory";
	
	/** the maximum size in megabytes string */
	public String size = "set max. backup size";

	/** the type of backup to perform */
	public int backupType = 0;
	
	/** the file filters to apply during the backup */
	public String filters = "all";

	/** Vector of the individual filters to apply */ 
	public Vector filterV = null;
	
	/** flag whether filters work subtracting or adding */
	public boolean filterOut = false;

	/** flag whether to try to start backup without user interaction */
	public boolean immediateStart = false;

	/** the result string or help-file of the backup process */
	public Object result;
	
	/** string indicating that the default value is to be used */
	protected static final String DEFAULT_INDICATOR = "_";

	/** string indicating that the default value is to be used */
	protected static final String AUTOSTART_INDICATOR = "start";
	
	/**
	 * Constructor parses settings from command line arguments and/or
	 * sets them to their default values
	 * 
	 * for all args[] use DEFAULT_INDICATOR (or nothing, if no more args[]
	 * follow) for default value
	 * 
	 * args[0] - source directory
	 * args[1] - backup directory
	 * args[2] - data directory
	 * args[3] - max. backup size in megabytes
	 * args[4] - backup type (1/2/3/4 incr/diff2diff/diff2full/full)
	 * args[5] - file filters, default "all"
	 * args[6] - autostart flag. If set to AUTOSTART_INDICATOR, try to start without user interaction.
	 * 
	 * @param args  the command line arguments to parse
	 */
	
	public GuiSettings (String[] args) {

		// parse source directory
		if (args.length > 0 && !args[0].equals(DEFAULT_INDICATOR) && new File(args[0]).isDirectory()) {
			source = args[0];
		}

		// parse backup directory
		if (args.length > 1 && !args[1].equals(DEFAULT_INDICATOR) && new File(args[1]).isDirectory()) {
			backupRoot = args[1];
		}
		
		// parse or set default data directory
		if (args.length > 2 && !args[2].equals(DEFAULT_INDICATOR) && new File(args[2]).isDirectory()) {
			dataRoot = args[2];
		} else {
			// default is ./data in installation directory
			File classPath = new File(System.getProperty("java.class.path"));
			// if classpath is a .jar file, take its parent directory
			if (!classPath.isDirectory()) classPath = classPath.getParentFile();
			dataRoot = Util.canonicalPath(classPath)+File.separator+"data";
		}

		// parse backup size string
		if (args.length > 3 && !args[3].equals(DEFAULT_INDICATOR)) {
			size = args[3];
		}
		
		// parse backup type (int)
		if (args.length > 4  && !args[4].equals(DEFAULT_INDICATOR)) {
			backupType = Util.parseInt(args[4]);
		}

		// parse file filters to apply
		if (args.length > 5  && !args[5].equals(DEFAULT_INDICATOR)) {
			filters = args[5];
		}
		
		// parse file filters to apply
		if (args.length > 6  && args[5].equals(AUTOSTART_INDICATOR)) {
			immediateStart = true;
		}
	}

	/**
	 * check the current settings for consistency
	 * 
	 * @return  true if backup is possible with settings
	 */
	public boolean check() {
		
		if (!new File(source    ).isDirectory()) return false;
		if (!new File(backupRoot).isDirectory()) return false;
		if (!new File(dataRoot  ).isDirectory()) return false;
		if (backupType == 0) return false;
		if (Util.parseInt(size) == 0) return false;
		filterV = filterCheck();
		return filterV != null;
	}

	/**
	 * check the current filters for consistency
	 * 
	 * @return  the filters Vector or null if inconsistent
	 */
	Vector filterCheck() {

		// check for default setting "all"
		if (filters.equals("all")) { return new Vector(); }
		
		try {

			// split filters into vector
			Vector v = new Vector(Arrays.asList(filters.split(" ")));
			
			// check whether to use additive or subtractive filtering
			if (v.get(0).equals("not")) {
				filterOut = true;
				v.remove(0);
			} else {
				filterOut = false;
			}

			// check whether all filters are valid regular expressions
			for (Enumeration e = v.elements(); e.hasMoreElements(); ) {
				"test.string".matches(e.nextElement().toString());
			}

			// filters are alright
			return v;
						
		} catch (Exception e) {
			// filters are not alright
			return null;
		}
	}
}
